"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.gitCreateJiraBranch = void 0;
const effect_1 = require("effect");
const Function_1 = require("effect/Function");
const Option_1 = require("effect/Option");
const git_client_1 = require("./git-client");
const jira_client_1 = require("./jira-client");
const environment_1 = require("./environment");
const types_1 = require("./types");
const gitCreateJiraBranch = (jiraKey, baseBranch, reset) => effect_1.Effect.gen(function* ($) {
    const [envProvider, gitClient, jiraClient] = yield* $(effect_1.Effect.all([environment_1.Environment, git_client_1.GitClient, jira_client_1.JiraClient]));
    const { defaultJiraKeyPrefix } = yield* $(envProvider.getEnv());
    const fullKey = buildJiraKey(defaultJiraKeyPrefix, jiraKey);
    const issue = yield* $(jiraClient.getJiraIssue(fullKey));
    const branchName = jiraIssueToBranchName(issue);
    const branchExists = yield* $(effect_1.Effect.map(gitClient.listBranches(), effect_1.Chunk.contains(branchName)));
    if (!reset && branchExists) {
        yield* $(gitClient.switchBranch(branchName));
        return (0, types_1.SwitchedBranch)(branchName);
    }
    const resetBranch = branchExists && reset;
    yield* $(effect_1.Option.match(baseBranch, {
        onNone: (0, Function_1.constant)(gitClient.createGitBranch),
        onSome: gitClient.createGitBranchFrom,
    })(branchName, resetBranch));
    return (resetBranch ? types_1.ResetBranch : types_1.CreatedBranch)(branchName);
});
exports.gitCreateJiraBranch = gitCreateJiraBranch;
const slugify = (str) => str
    .toLowerCase()
    .replace(/ä/g, 'ae')
    .replace(/ö/g, 'oe')
    .replace(/ü/g, 'ue')
    .replace(/ß/g, 'ss')
    .replace(/[^a-zA-Z0-9]/g, '-')
    .replace(/-+/g, '-')
    .replace(/^-|-$/g, '');
const jiraIssueToBranchName = (issue) => {
    const branchtype = jiraIssuetypeBranchtype(issue.fields.issuetype);
    return `${branchtype}/${issue.key}-${slugify(issue.fields.summary)}`;
};
const jiraIssuetypeBranchtype = (issuetype) => {
    if (issuetype.name.match(/bug/i)) {
        return 'fix';
    }
    return 'feat';
};
const buildJiraKey = (defaultJiraKeyPrefix, jiraKey) => jiraKey.match(/^([a-z]+)-(\d+)$/i) || (0, Option_1.isNone)(defaultJiraKeyPrefix)
    ? jiraKey
    : `${defaultJiraKeyPrefix.value}-${jiraKey}`;
